/**
@license
Copyright 2020 Google LLC


Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import Vue from "vue";
import PortalVue from "portal-vue";
import VueScrollTo from "vue-scrollto";
import App from "./App.vue";
import store from "./store/index";
import router from "./router";
import pm from "./pages";

import { ScormException } from "@/lib/exceptions";
import { interruptionAlert, launchAlert } from "@/alerts";

// SCORM schemas
if (process.env.VUE_APP_SCORM === "scorm12") require("./scorm12");
if (process.env.VUE_APP_SCORM === "scorm2004") require("./scorm2004");

// global styles
import "@/assets/styles/main.pcss";

// common components
import BaseLayout from "./layouts/BaseLayout";
import PageLayout from "./layouts/PageLayout";
import TitleLayout from "./layouts/TitleLayout";
import Instruction from "./components/Instruction";
import Note from "./components/Note";

const pageMap = pm.getPageMap();

Vue.config.productionTip = false;
Vue.use(VueScrollTo);

// Common components
Vue.component("base-layout", BaseLayout);
Vue.component("page-layout", PageLayout);
Vue.component("title-layout", TitleLayout);
Vue.component("instruction", Instruction);
Vue.component("note", Note);

// create components for each page
pageMap.forEach((component, name) => {
  Vue.component(name, component);
});

router.beforeEach((to, from, next) => {
  store.commit("SET_LOADING", true);
  next();
});
router.afterEach((to, from) => {
  store.commit("SET_LOADING", false);
});

// Vue use
Vue.use(PortalVue);

function launchApp() {
  new Vue({
    router,
    store,
    created() {
      store.subscribe(() => {
        try {
          store.dispatch("storage/saveState");
        } catch (e) {
          console.error(e);
          if (e instanceof ScormException) {
            interruptionAlert();
            this.disableRouting();
          }
        }
      });
    },
    methods: {
      disableRouting() {
        router.beforeEach((to, from, next) => {
          interruptionAlert();
          next(false);
        });
      }
    },
    render: h => h(App, { ref: "app" })
  }).$mount("#app");
}

// Handle persistant storage, then create the Vue instance
try {
  store.dispatch("storage/recoverState").then(() => {
    launchApp();
  });
} catch (e) {
  console.error(e);
  if (e instanceof ScormException) launchAlert();
}
