/**
@license
Copyright 2020 Google LLC


Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import VueComponent from "vue";
import VueScrollTo from "vue-scrollto";
import { outOfView, isFixed } from "@/lib/scroll";
import { findFirstFocusable, isFocusable } from "@/lib/focus";

const DEBUG = false;
const SCROLL_DURATION = 500;
const SCROLL_OPTIONS = {
  container: "body",
  duration: SCROLL_DURATION,
  easing: "ease-in",
  offset: -16,
  force: true,
  cancelable: false,
  onStart: false,
  onDone: false,
  onCancel: false,
  x: false,
  y: true
};

function focusNode(node) {
  if (node instanceof HTMLElement) {
    const focusable = isFocusable(node) ? node : findFirstFocusable(node);
    if (focusable) focusable.focus();
  }
}

function lastMatching(element, selector) {
  const nodes = element.querySelectorAll(selector);
  if (nodes.length) {
    return nodes[nodes.length - 1];
  }
  return undefined;
}

const scrollMixin = {
  methods: {
    scrollElementForTarget(target) {
      let el;
      if (DEBUG) {
        console.log(`scrollElementForTarget ${target}`);
        console.dir(target);
      }
      if (typeof target === "string") {
        if (this.$el.matches(target)) el = this.$el;
        else el = lastMatching(this.$el, target);
        return el;
      }

      if (Array.isArray(target)) el = target.pop();
      else el = target;
      if (el instanceof VueComponent) return el.$el;

      if (el instanceof Element) return el;
      return null;
    },
    async scrollToView(target, options = {}) {
      // wait for dynamically rendered elements be available in the DOM
      await this.$nextTick();
      const focus = options.focus || false;
      const el = this.scrollElementForTarget(target);
      if (el && !isFixed(el)) {
        const delta = outOfView(el);
        if (delta) window.scrollBy({ top: delta, behavior: "smooth" });
        if (focus) focusNode(el);
      }
      if (DEBUG) {
        console.log("scrollToView");
        console.dir(el);
      }
    },
    async scrollToTarget(target, options = {}) {
      await this.$nextTick();
      const opts = Object.assign(SCROLL_OPTIONS, options);
      if (DEBUG) console.log(`scroll(${options})`);
      const el = this.scrollElementForTarget(target);
      if (el) {
        if (outOfView(el)) VueScrollTo.scrollTo(el, SCROLL_DURATION, opts);
      }
      if (DEBUG) {
        console.log("scrollToTarget");
        console.dir(el);
      }
    }
  }
};

export default scrollMixin;
