/**
@license
Copyright 2020 Google LLC


Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import Vue from "vue";
import Vuex from "vuex";
import _ from "lodash";
import pm from "@/pages";
import mcq from "./modules/mcq";
import stepper from "./modules/stepper";
import storage from "./modules/storage";

const DEBUG = false;

Vue.use(Vuex);

export default new Vuex.Store({
  modules: { mcq, stepper, storage },
  state: {
    exists: true, // used to check if state is stored
    bookmark: 0, // highest page number
    pageNo: 0, // page number; valid > 0
    data: {}, // page data
    values: {}, // whatever
    forms: {},
    loading: false
  },
  getters: {
    bookmark: state => (state.bookmark ? pm.getRoute(state.bookmark) : ""),
    data: state => state.data,
    page: state => pm.getName(state.pageNo),
    pageNo: state => state.pageNo,
    backPage: state => {
      if (state.pageNo < 1) return 0;
      return state.pageNo - 1;
    },
    nextPage: state => {
      if (state.pageNo < pm.numPages()) return state.pageNo + 1;
      else return 0;
    },
    getPageData: state =>
      function(page) {
        const n = pm.getNumber(page);
        if (n && typeof state.data[n] === "object") return state.data[n];
        return {};
      },
    getValue: state =>
      function(name) {
        return state.values[name];
      },
    getForm: state =>
      function(name) {
        return state.forms[name];
      }
  },
  mutations: {
    SET_LOADING(state, isLoading) {
      state.loading = isLoading;
    },
    SET_PAGE(state, page = "") {
      let n = pm.getNumber(page);
      // window.console.log(`SET_PAGE ${page} => ${n}`);
      state.pageNo = n;
      if (n > 1 && state.pageNo > state.bookmark) {
        state.bookmark = state.pageNo;
        // window.console.log(`update bookmark => ${state.bookmark}`);
      }
    },
    SET_PAGE_DATA(state, { pageNo, pageData }) {
      const n = pm.getNumber(pageNo);
      if (n) Vue.set(state.data, n, pageData);
    },
    SET_VALUE(state, payload) {
      if (!payload.name || undefined === payload.value) {
        throw `Invalid SET_VALUE payload: ${JSON.stringify(payload)}`;
      }
      state.values[payload.name] = payload.value;
    },
    SET_FORM(state, payload) {
      if (!payload.name || undefined === payload.value) {
        throw `Invalid SET_FORM payload: ${JSON.stringify(payload)}`;
      }
      state.forms[payload.name] = payload.value;
    },
    CLEAR_PAGE(state) {
      if (typeof state !== "object") throw `Unexpected state ${state}`;
      state.pageNo = 0;
    },
    RESET_STORE(state) {
      state.bookmark = 0;
      state.pageNo = 0;
      state.data = {};
      state.values = {};
      state.forms = {};
    },
    REPLACE_STATE(_, stored) {
      if (Object.keys(stored).length) this.replaceState(stored);
    }
  },
  actions: {
    setPageData({ getters, commit }, { pageData }) {
      try {
        const pageNo = getters.pageNo;
        if (pageNo === undefined || typeof pageData !== "object") {
          throw new Error(`Invalid setPageData parameters. pageData: ${JSON.stringify(pageData)}`);
        }
        commit("SET_PAGE_DATA", { pageNo, pageData });
      } catch (msg) {
        throw new Error(msg);
      }
    },
    reset({ commit, dispatch }) {
      commit("RESET_STORE");
      dispatch("mcq/resetAll");
      dispatch("stepper/resetAll");
    }
  }
});
