/**
@license
Copyright 2020 Google LLC


Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import Vue from "vue";

function validateId(id) {
  if (typeof id !== "string") throw new Error(`invalid form id ${id}`);
}

function validateChoices(choices) {
  if (!Array.isArray(choices) || choices.filter(choice => choice.value === undefined).length > 0) {
    console.dir(choices);
    throw new Error(`invalid form choices ${choices}`);
  }
}

const template = {
  inputState: {},
  submitted: false,
  submittable: false,
  icons: false,
  correctness: 0
};

export default {
  namespaced: true,
  state: {
    form: {},
    icon: {
      default: { class: "", label: "" },
      correct: { class: "fas fa-arrow-right", label: "correct" },
      incorrect: { class: "fas fa-times", label: "incorrect" }
    }
  },
  getters: {
    isSubmitted: state =>
      function(id) {
        return state.form[id] && state.form[id].submitted;
      },
    isSubmittable: state =>
      function(id) {
        return state.form[id] && state.form[id].submittable;
      },
    correctness: state =>
      function(id) {
        if (!state.form[id]) return 0;
        return state.form[id].correctness;
      },
    isCorrect: state =>
      function(id) {
        return state.form[id] && state.form[id].correctness > 0;
      },
    isIncorrect: state =>
      function(id) {
        return state.form[id] && state.form[id].correctness < 0;
      },
    isValue: state =>
      function(id, value) {
        return state.form[id] && state.form[id].inputState[value];
      },
    isCheckedValue: state =>
      function(id, value) {
        try {
          return state.form[id] && state.form[id].inputState[value].checked;
        } catch (msg) {
          return false;
        }
      },
    isExaminedValue: state =>
      function(id, value) {
        try {
          return state.form[id] && state.form[id].inputState[value].examined;
        } catch (msg) {
          return undefined;
        }
      },
    values: state =>
      function(id) {
        return Object.keys(state.form[id].inputState);
      },
    checkedValues: state =>
      function(id) {
        if (!state.form[id]) return [];
        return Object.keys(state.form[id].inputState).filter(
          value => state.form[id].inputState[value].checked
        );
      },
    examinedValues: state =>
      function(id) {
        if (!state.form[id]) return [];
        return Object.keys(state.form[id].inputState).filter(
          value => state.form[id].inputState[value].examined
        );
      },
    iconForValue: state =>
      function(id, value) {
        if (!state.form[id] || !state.form[id].inputState[value]) return state.icon.default;
        const input = state.form[id].inputState[value];
        if (!input.correctness) return state.icon.default;
        return input.correctness > 0 ? state.icon.correct : state.icon.incorrect;
      },
    useIcons: state =>
      function(id) {
        if (!state.form[id]) return false;
        return state.form[id].icons;
      },
    inputAttribute: state =>
      function(id, value, attributeName) {
        try {
          return state.form[id] && state.form[id].inputState[value][attributeName];
        } catch (msg) {
          return undefined;
        }
      }
  },
  mutations: {
    NEW_FORM(state, { id }) {
      if (id) {
        validateId(id);
        if (!state.form[id]) Vue.set(state.form, id, JSON.parse(JSON.stringify(template)));
      }
    },
    RESET_INPUTS(state, { id, choices }) {
      validateId(id);
      validateChoices(choices);
      try {
        for (let choice of choices) {
          Vue.set(state.form[id].inputState, choice.value, {
            checked: false,
            examined: false
          });
        }
      } catch (msg) {
        throw new Error(msg);
      }
    },
    RESET_ALL_INPUTS(state) {
      state.form = {};
    },
    EXAMINE_INPUT(state, { id, value }) {
      try {
        state.form[id].inputState[value].examined = true;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    CHECK_INPUT(state, { id, value }) {
      try {
        state.form[id].inputState[value].checked = true;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    UNCHECK_INPUT(state, { id, value }) {
      try {
        state.form[id].inputState[value].checked = false;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    UNCHECK_ALL(state, { id }) {
      try {
        for (let value of Object.keys(state.form[id].inputState))
          state.form[id].inputState[value].checked = false;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    SET_SUBMITTED(state, { id, submitted }) {
      if (state.form[id]) state.form[id].submitted = !!submitted;
      else throw new Error(`no form ${id}`);
    },
    SET_SUBMITTABLE(state, { id, submittable }) {
      if (state.form[id]) state.form[id].submittable = !!submittable;
      else throw new Error(`no form ${id}`);
    },
    SET_CORRECTNESS(state, { id, correctness }) {
      if (state.form[id]) state.form[id].correctness = correctness;
      else throw new Error(`no form ${id}`);
    },
    SET_USE_ICONS(state, { id }) {
      if (state.form[id]) state.form[id].icons = true;
    },
    SET_INPUT_ATTRIBUTE(state, { id, value, attribute }) {
      if (!state.form[id]) throw new Error(`no form ${id}`);
      if (typeof attribute !== "object" || !attribute.name)
        throw new Errror(`SET_INPUT_ATTRIBUTE ${id} ${value}: invalid attribute`);
      Vue.set(state.form[id].inputState[value], attribute.name, attribute.value);
    }
  },
  actions: {
    setup({ state, commit }, { id, choices, submittable, icons }) {
      if (id && !state.form[id]) {
        commit("NEW_FORM", { id });
        commit("RESET_INPUTS", { id, choices });
        if (submittable) commit("SET_SUBMITTABLE", { id, submittable: true });
        if (icons) commit("SET_USE_ICONS", { id });
      }
    },
    reset({ commit }, { id, choices }) {
      commit("RESET_INPUTS", { id, choices });
    },
    resetAll({ commit }) {
      commit("RESET_ALL_INPUTS");
    },
    change({ commit }, { id, value, checked, clear }) {
      if (clear) commit("UNCHECK_ALL", { id });
      if (checked) commit("CHECK_INPUT", { id, value });
      else commit("UNCHECK_INPUT", { id, value });
      commit("SET_SUBMITTABLE", { id, submittable: true });
      commit("SET_SUBMITTED", { id, submitted: false });
    },
    submit({ state, commit, dispatch }, { id, button }) {
      const values = Object.keys(state.form[id].inputState);
      for (let value of values) {
        if (state.form[id].inputState[value].checked) commit("EXAMINE_INPUT", { id, value });
      }
      commit("SET_SUBMITTED", { id, submitted: true });
      dispatch("disable", { id, button });
    },
    tryAgain({ state, commit }, { id }) {
      commit("SET_SUBMITTED", { id, submitted: false });
      commit("SET_SUBMITTABLE", { id, submittable: false });
    },
    setCorrectness({ commit }, { id, correct }) {
      commit("SET_CORRECTNESS", { id, correctness: correct ? 1 : -1 });
    },
    setInputAttribute({ commit }, { id, value, attribute }) {
      commit("SET_INPUT_ATTRIBUTE", { id, value, attribute });
    },
    disable({ state, commit }, { id, button }) {
      const promise = new Promise(function(resolve) {
        if (button instanceof HTMLElement) {
          if (button === document.activeElement)
            button.onblur = () => {
              resolve(100);
            };
          else resolve(50);
        } else resolve(0);
      });
      promise.then(function(timeout) {
        setTimeout(() => {
          commit("SET_SUBMITTABLE", { id, submittable: false });
        }, timeout);
      });
    }
  }
};
