/**
@license
Copyright 2020 Google LLC


Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import Vue from "vue";
import _ from "lodash";

function validateId(id) {
  if (!id || typeof id !== "string") throw new Error(`invalid form id ${id}`);
}

const defaults = {
  numberSteps: 0,
  enabledStep: 0,
  advanceDisabled: true
};

export default {
  namespaced: true,
  state: { stepper: {} },
  getters: {
    enabledStep: state =>
      function enabledStep(id) {
        return state.stepper[id] ? state.stepper[id].enabledStep : defaults.enabledStep;
      },
    disabled: state =>
      function disabled(id) {
        return state.stepper[id] ? state.stepper[id].advanceDisabled : defaults.advanceDisabled;
      }
  },
  mutations: {
    NEW_STEPPER(state, { id, nsteps }) {
      try {
        validateId(id);
        if (!Number(nsteps) || Number(nsteps) < 1) throw new Error(`stepper ${id} has ${nsteps}!`);
        if (!state.stepper[id]) Vue.set(state.stepper, id, _.clone(defaults));
        state.stepper[id].numberSteps = nsteps;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    START(state, { id }) {
      try {
        state.stepper[id].enabledStep = 1;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    ADVANCE(state, { id }) {
      try {
        const stepper = state.stepper[id];
        if (!stepper.advanceDisabled && stepper.enabledStep < stepper.numberSteps)
          stepper.enabledStep++;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    ENABLE_ADVANCE(state, { id }) {
      try {
        state.stepper[id].advanceDisabled = false;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    DISABLE_ADVANCE(state, { id }) {
      try {
        state.stepper[id].advanceDisabled = true;
      } catch (msg) {
        throw new Error(msg);
      }
    },
    RESET_ALL_STEPPERS(state) {
      state.stepper = {};
    }
  },
  actions: {
    setup({ state, commit }, { id, nsteps }) {
      try {
        if (!state.stepper[id]) {
          commit("NEW_STEPPER", { id, nsteps });
          commit("START", { id });
        }
      } catch (msg) {
        console.error(msg);
      }
    },
    restart({ commit }, { id }) {
      try {
        commit("START", { id });
      } catch (msg) {
        console.error(msg);
      }
    },
    enable({ commit }, { id }) {
      try {
        commit("ENABLE_ADVANCE", { id });
      } catch (msg) {
        console.error(msg);
      }
    },
    disable({ commit }, { id }) {
      try {
        commit("DISABLE_ADVANCE", { id });
      } catch (msg) {
        console.error(msg);
      }
    },
    advance({ commit }, { id }) {
      try {
        commit("ADVANCE", { id });
      } catch (msg) {
        console.error(msg);
      }
    },
    resetAll({ commit }) {
      commit("RESET_ALL_STEPPERS");
    }
  }
};
